/********************************************************
 * Data Visualization - 2D Rough Draft                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Hannah Wolfe                                 *
 * Professor: George Legrady                            *
 *           
 *
 ********************************************************/

// Declare variables
Table table;
entry [] rows;
ArrayList<String> authorList = new ArrayList<String>();
int numCols;
float cellWidth, cellHeight, yearWidth;
float constantVerMargin = 50;
float constantHorMargin = 50;
int max = 0;
int startYear = 2006;
boolean normalization = false;
int colorCode = 0;
boolean reorder = false;
String[] types = {"item", "medium", "genre", "cd/cas", "book", "sci", "mys", "wes", "fic", "ya"};
int typeP = 0;
String letter = " ";

//Only run one time
// In Processing 3, the size() function must be the first line of code inside setup()
// and needs to use numbers instead of variables. 
void setup() {
  size(1920, 1080); // setup the size of the window
  //background(random(255), random(255), random(255));
  
  table = loadTable("output.csv", "header");
  // order of rows and cols depends on orientation
  numCols = table.getRowCount();
  rows = new entry[numCols-1];
  int a = 0;
  for (int i = 0; i < numCols-1; i++) {
    String itemType = table.getString(i+1,8);
    String mediumType = table.getString(i+1,9);
    boolean returned = boolean(table.getInt(i+1,4));
    int checkoutDay = table.getInt(i+1,12);
    int checkinDay = table.getInt(i+1,13);
    String author = table.getString(i+1,10);
    if (checkoutDay > max) {
      max = checkoutDay;
    }
 //   println(b_returned[i]+ ":" + b_itemType[i]+ ":" +b_checkoutDay[i]+":" +b_checkinDay[i]);
    
    addAuthor(author, authorList, a);

    rows[i] = new entry(returned, a, checkoutDay, checkinDay, itemType, mediumType, author);
  }
  
  println(max);
}

// Refresh the canvas every frame, default is 24 fps
// all sketching and interactive elements are added to Draw
void draw() {  
  // println(hour() + ":" + minute() + ":" + second());
  ArrayList<String> authorList = new ArrayList<String>();
  background(250);
  surface.setResizable(true);
  
  int a = 0;
  for(int i=0; i<numCols-1; i++)
  {
    if (!rows[i].filtered(types[typeP], letter)) {
      a = addAuthor(rows[i].author, authorList, a);
      rows[i].author2 = a;
    }
  }
  
  cellWidth = (width - 2*constantVerMargin)/max*21;
  yearWidth = (width - 2*constantVerMargin)/max*365/2;
  cellHeight = (height - 2*constantHorMargin)/(authorList.size()*2);
  
  for(int i=0; i<numCols-1; i++)
  {
  //  println("hear me");
    if (!rows[i].filtered(types[typeP], letter)) {
      color c = rows[i].getColor(types[typeP]);
      strokeWeight(1);
      stroke(c);
      fill(c);
      if (rows[i].returned) {
        rect(
          constantHorMargin+cellWidth*rows[i].checkoutDay/21,
          constantVerMargin+cellHeight*rows[i].author2*2,
          cellWidth*max((rows[i].checkinDay-rows[i].checkoutDay)/21, 1),
          cellHeight);
      } else {
        ellipse(
          constantHorMargin+cellWidth*rows[i].checkoutDay/21,
          constantVerMargin+cellHeight*rows[i].author2*2+cellHeight*.5,
          cellWidth,
          cellWidth);
      }
    }
  }
 
  String ss = "";
  for( int i=0; i<authorList.size(); i++)
  {
    textAlign(RIGHT, CENTER);
    textSize(10);
    fill(0);
    String aa = "";
    if (authorList.get(i).length() > 1) {
      aa = authorList.get(i).substring(0,1);
    }
    if (!ss.equals(aa)) {
      text(aa, constantHorMargin, constantVerMargin+cellHeight*i*2);
      ss = aa;
    }  
  }
  
  float x = constantHorMargin*1.25;
  int year = 2005;
  while (x < width) {
    text(year, x, height-constantVerMargin);
    x = x + yearWidth*2;
    year++;
  }
  
  
  fill(0);
  textSize(14);
  textAlign(CENTER, CENTER);
  text("LOST AND FORGOTTEN BOOKS", width/2, constantVerMargin*.5);
  textSize(10);
  textAlign(CENTER, CENTER);
  text("[ Press letters to filter by author name, Press numbers to filter by type ]",  width/2, height-constantVerMargin*.45);
  
  //add author name
  String name = " ";
  int mY = mouseY;
  int mX = mouseX;
  if (mY > constantVerMargin && mY < height-constantVerMargin) {
    if (authorList.size() > 0) {
      name = authorList.get(floor((mY-constantVerMargin)/(cellHeight*2)));
    }
  }
  textSize(14);
  textAlign(TOP, LEFT);
  text(name, mX-2, mY-2);
  
  //add legend
  switch(types[typeP]) {
    case "item":
      drawLegend("Genre","Y.A.","Fiction");
      break;
    case "medium":
      drawLegend("Tape","CD","Book");
      break;
    case "genre":
      drawLegend("Sci-Fic","Western","Mystery");
      break;
    case "cd/cas":
      drawLegend("Tape","CD","");
      break;
    case "book":
      drawLegend("Book","","");
      break;
    case "sci":
      drawLegend("Sci-Fic","","");
      break;
    case "mys":
      drawLegend("Mystery","","");
      break;
    case "wes":
      drawLegend("Western","","");
      break;
    case "fic":
      drawLegend("Fiction","","");
      break;
    case "ya":
      drawLegend("Y.A.","","");
      break;
  }
  
}

//CTRL-SPACE for auto-completion